/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.binding.gateway.clientserver.api.AdminRuntimeService;
import org.ow2.petals.binding.gateway.clientserver.api.AdminService;
import org.ow2.petals.binding.gateway.clientserver.api.CommonAdminService;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;

/**
 * The command 'bc-gateway.add-transport-listener'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AddTransportListener extends BcGatewayAbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "add-transport-listener";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "Add a new transport listener to the given BC Gateway";

    /**
     * The option associated to the transport listener identifier
     */
    public static final String IDENTIFIER_SHORT_OPTION = "i";

    private static final String IDENTIFIER_LONG_OPTION = "identifier";

    private static final String IDENTIFIER_ARG_NAME = "transport-listener-identifier";

    private static final Option IDENTIFIER_OPTION = Option.builder(IDENTIFIER_SHORT_OPTION)
            .longOpt(IDENTIFIER_LONG_OPTION).required().numberOfArgs(1).argName(IDENTIFIER_ARG_NAME)
            .desc("The unique identifier of the transport listener to add to the BC Gateway.").build();

    /**
     * The option associated to the transport listener port
     */
    public static final String PORT_SHORT_OPTION = "p";

    private static final String PORT_LONG_OPTION = "port";

    private static final String PORT_ARG_NAME = "listening-port";

    private static final Option PORT_OPTION = Option.builder(PORT_SHORT_OPTION).longOpt(PORT_LONG_OPTION).required()
            .numberOfArgs(1).argName(PORT_ARG_NAME)
            .desc("The listening port of the transport listener to add to the BC Gateway.").build();

    public AddTransportListener() {
        super(NAME, DESCRIPTION);
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(PORT_OPTION).addOption(IDENTIFIER_OPTION);
    }

    @Override
    protected void doExecute(final CommandLine cli, final String componentId) throws CommandException {
        try {
            final String portStr = cli.getOptionValue(PORT_SHORT_OPTION);
            final int port;
            try {
                port = Integer.parseInt(portStr);
            } catch (final NumberFormatException e) {
                throw new CommandInvalidArgumentException(this, PORT_OPTION, portStr, e);
            }

            CommonAdminService adminServiceClient;
            final JMXClient client = JMXClientConnection.getJMXClient();
            try {
                // First, try using the installation configuration component
                adminServiceClient = client.getInstallerComponentClient(componentId)
                        .getInstallationConfigurationClient(AdminService.class);
            } catch (final NullPointerException | InstallerComponentDoesNotExistException
                    | ConfigurationComponentDoesNotExistException e) {
                // 2nd, try using the runtime configuration component
                adminServiceClient = client.getComponentClient(componentId)
                        .getRuntimeConfigurationClient(AdminRuntimeService.class);
            }

            adminServiceClient.addTransportListener(cli.getOptionValue(IDENTIFIER_SHORT_OPTION), port);

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }
}
