/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import java.util.Collections;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.binding.gateway.clientserver.api.AdminRuntimeService;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;

public class Reconnect extends BcGatewayAbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "reconnect";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "Triggers reconnection of client domains of the BC Gateway";

    /**
     * The option associated to the service unit identifier
     */
    public static final String SU_ID_SHORT_OPTION = "s";

    private static final String SU_ID_LONG_OPTION = "service-unit";

    private static final String SU_ID_ARG_NAME = "service-unit-identifier";

    private static final Option SU_ID_OPTION = Option.builder(SU_ID_SHORT_OPTION).longOpt(SU_ID_LONG_OPTION)
            .required(false).numberOfArgs(1).argName(SU_ID_ARG_NAME)
            .desc("The service unit identifier deployed on the BC Gateway.").build();

    /**
     * The option associated to the provider domain identifier
     */
    public static final String PD_ID_SHORT_OPTION = "c";

    private static final String PD_ID_LONG_OPTION = "provider-domain";

    private static final String PD_ID_ARG_NAME = "provider-domain-identifier";

    private static final Option PD_ID_OPTION = Option.builder(PD_ID_SHORT_OPTION).longOpt(PD_ID_LONG_OPTION)
            .required(false).numberOfArgs(1).argName(PD_ID_ARG_NAME)
            .desc("The provider domain identifier defined in the specified service unit.").build();

    /**
     * The option associated to forcing the reconnect
     */
    public static final String FORCE_SHORT_OPTION = "f";

    private static final String FORCE_LONG_OPTION = "force";

    private static final Option FORCE_OPTION = Option.builder(FORCE_SHORT_OPTION).longOpt(FORCE_LONG_OPTION)
            .required(false).numberOfArgs(0)
            .desc("Force reconnection even if the provider domain is already connected.").build();

    public Reconnect() {
        super(NAME, DESCRIPTION);
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(SU_ID_OPTION).addOption(PD_ID_OPTION).addOption(FORCE_OPTION);
    }

    @Override
    protected void doExecute(final CommandLine cli, final String componentId) throws CommandException {
        try {

            final String suId;
            if (cli.hasOption(SU_ID_SHORT_OPTION)) {
                suId = cli.getOptionValue(SU_ID_SHORT_OPTION);
            } else {
                suId = null;
            }

            final String pdId;
            if (cli.hasOption(PD_ID_SHORT_OPTION)) {
                if (suId == null) {
                    // TODO explain?!
                    throw new CommandMissingOptionsException(this, Collections.singletonList(SU_ID_SHORT_OPTION));
                }
                pdId = cli.getOptionValue(PD_ID_SHORT_OPTION);
            } else {
                pdId = null;
            }

            final boolean force = cli.hasOption(FORCE_SHORT_OPTION);

            final JMXClient client = JMXClientConnection.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);
            if (suId != null) {
                if (pdId != null) {
                    admin.reconnectDomains(suId, pdId, force);
                } else {
                    admin.reconnectDomains(suId, force);
                }
            } else {
                admin.reconnectDomains(force);
            }

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }
}
