/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import java.util.Collections;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.binding.gateway.clientserver.api.AdminRuntimeService;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;

/**
 * The command 'bc-gateway.repropagate'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class Repropagate extends BcGatewayAbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "repropagate";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "Repropagate all elected endpoints to the client domains of the BC Gateway";

    /**
     * The option associated to the service unit identifier
     */
    public static final String SU_ID_SHORT_OPTION = "s";

    private static final String SU_ID_LONG_OPTION = "service-unit";

    private static final String SU_ID_ARG_NAME = "service-unit-identifier";

    private static final Option SU_ID_OPTION = Option.builder(SU_ID_SHORT_OPTION).longOpt(SU_ID_LONG_OPTION)
            .required(false).numberOfArgs(1).argName(SU_ID_ARG_NAME)
            .desc("The service unit identifier deployed on the BC Gateway.").build();

    /**
     * The option associated to the consumer domain identifier
     */
    public static final String CD_ID_SHORT_OPTION = "c";

    private static final String CD_ID_LONG_OPTION = "consumer-domain";

    private static final String CD_ID_ARG_NAME = "consumer-domain-identifier";

    private static final Option CD_ID_OPTION = Option.builder(CD_ID_SHORT_OPTION).longOpt(CD_ID_LONG_OPTION)
            .required(false).numberOfArgs(1).argName(CD_ID_ARG_NAME)
            .desc("The consumer domain identifier defined in the specified service unit.").build();

    public Repropagate() {
        super(NAME, DESCRIPTION);
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(SU_ID_OPTION).addOption(CD_ID_OPTION);
    }

    @Override
    protected void doExecute(final CommandLine cli, final String componentId) throws CommandException {
        try {

            final String suId;
            if (cli.hasOption(SU_ID_SHORT_OPTION)) {
                suId = cli.getOptionValue(SU_ID_SHORT_OPTION);
            } else {
                suId = null;
            }
            
            final String cdId;
            if (cli.hasOption(CD_ID_SHORT_OPTION)) {
                if (suId == null) {
                    // TODO explain?!
                    throw new CommandMissingOptionsException(this, Collections.singletonList(SU_ID_SHORT_OPTION));
                }
                cdId = cli.getOptionValue(CD_ID_SHORT_OPTION);
            } else {
                cdId = null;
            }
            
            final JMXClient client = JMXClientConnection.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);
            if (suId != null) {
                if (cdId != null) {
                    admin.refreshPropagations(suId, cdId);
                } else {
                    admin.refreshPropagations(suId);
                }
            } else {
                admin.refreshPropagations();
            }

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }
}
