/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.bcgateway.mock.PetalsBcGatewayRuntimeConfigurationMock;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;


/**
 * Unit tests of the command 'bc-gateway.reconnect'
 * 
 * @author Victor Noel - Linagora
 * 
 */
public class ReconnectTest extends BcGatewayAbstractCommandTest {

    @Before
    public void setUp() {
        this.command = new Reconnect();
    }

    /**
     * <p>
     * Test of the command "bc-gateway.reconnect" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, reconnect is done on the component identified by "petals-bc-jb-gateway".
     * </p>
     */
    @Test
    public void noArgumentProvided() throws Exception {

        // Set the environment
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient("component-B", ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        command.execute(new String[] {});

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertTrue("Default component not reconnected",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(null, null, false));
        assertFalse("Another component was reconnected",
                petalsBcGatewayRuntimeConfigurationMockOther.wasReconnectInvoked());
    }

    /**
     * <p>
     * Test of the command "bc-gateway.reconnect" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid service unit identifier is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, reconnect is done on the component identified by "petals-bc-jb-gateway" and
     * the SU identified by the provided identifier.
     * </p>
     */
    @Test
    public void suIdentifierProvided() throws Exception {

        // Set the environment
        final String aSuIdentifier = "su-345-test";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient("component-B", ComponentType.BINDING, null, null,
                petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        command.execute(new String[] { "-" + Reconnect.SU_ID_SHORT_OPTION, aSuIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertFalse("Default component reconnected for all",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(null, null, null));
        assertTrue("Default component's SU not reconnected",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(aSuIdentifier, null, false));
        assertFalse("Another component was reconnected",
                petalsBcGatewayRuntimeConfigurationMockOther.wasReconnectInvoked());
    }

    /**
     * <p>
     * Test of the command "bc-gateway.reconnect" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid provider domain identifier is provided</li>
     * <li>the option '-s' is missing</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void providerDomainIdentifierProvidedButServiceUnitIdentifierMissing() throws Exception {

        // Set the environment
        final String pdId = "pd-id-valid";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient("component-B", ComponentType.BINDING, null, null,
                petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        
        exceptions.expect(CommandMissingOptionsException.class);

        try {
            command.execute(new String[] { "-" + Reconnect.PD_ID_SHORT_OPTION, pdId });
        } catch (final CommandMissingOptionsException e) {
            assertTrue(e.getMissingOptions().contains(Reconnect.SU_ID_SHORT_OPTION));
            assertEquals(1, e.getMissingOptions().size());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.reconnect" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid service unit identifier is provided</li>
     * <li>a valid provider domain identifier is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, reconnect is done on the component identified by "petals-bc-jb-gateway" and
     * the consumer domain identified by the provided identifier.
     * </p>
     */
    @Test
    public void suIdentifierAndProviderDomainIdentifielProvided() throws Exception {

        // Set the environment
        final String aSuIdentifier = "su-345-test";
        final String pdId = "pd-id-valid";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient("component-B", ComponentType.BINDING, null, null,
                petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        command.execute(new String[] { "-" + Reconnect.SU_ID_SHORT_OPTION, aSuIdentifier,
                "-" + Reconnect.PD_ID_SHORT_OPTION, pdId });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertFalse("Default component reconnected for all",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(null, null, null));
        assertFalse("Default component's SU reconnected for all",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(aSuIdentifier, null, null));
        assertTrue("Default component's SU's consumer domain not reconnected",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked(aSuIdentifier, pdId, false));
        assertFalse("Another component was reconnected",
                petalsBcGatewayRuntimeConfigurationMockOther.wasReconnectInvoked());
    }

    /**
     * <p>
     * Test of the command "bc-gateway.reconnect" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid component identifier is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, refresh is done on the component identified by the given identifier.
     * </p>
     */
    @Test
    public void componentIdentifierProvided() throws Exception {

        // Set the environment
        final String anotherComponentIdentifier = "component-B";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient(anotherComponentIdentifier, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        command.execute(new String[] { "-" + Reconnect.COMPONENT_ID_SHORT_OPTION, anotherComponentIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertFalse("Default component refreshed",
                petalsBcGatewayRuntimeConfigurationMockDefault.wasReconnectInvoked());
        assertTrue("Other component not refreshed",
                petalsBcGatewayRuntimeConfigurationMockOther.wasReconnectInvoked(null, null, false));
    }
}
