/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import java.net.UnknownHostException;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.bcgateway.mock.PetalsBcGatewayInstallationMock;
import org.ow2.petals.cli.extension.bcgateway.mock.PetalsBcGatewayRuntimeConfigurationMock;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;


/**
 * Unit tests of the command 'bc-gateway.set-transport-listener-port'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class SetTransportListenerPortTest extends BcGatewayAbstractCommandTest {

    @Before
    public void setUp() {
        this.command = new SetTransportListenerPort();
    }

    @Override
    public void componentIdentifierOptionMissing() throws Exception {
        // disable because it makes no sense here: this commands has required options so the wrong exception will be
        // thrown
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void noArgumentProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, SetTransportListenerPort.IDENTIFIER_SHORT_OPTION,
                    SetTransportListenerPort.PORT_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>only the option about transport listener identifier is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void onlyIdentifierProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + SetTransportListenerPort.IDENTIFIER_SHORT_OPTION, "an-identifier" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, SetTransportListenerPort.PORT_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>only the option about transport listener port is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void onlyPortProvided() throws ConnectionErrorException, DuplicatedServiceException, MissingServiceException,
            CommandException, InstallationServiceErrorException, UnknownHostException, HostMissingException,
            InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + SetTransportListenerPort.PORT_SHORT_OPTION, "40000" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, SetTransportListenerPort.IDENTIFIER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option about transport listener identifier is provided</li>
     * <li>the option about transport listener port is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid port value.
     * </p>
     */
    @Test
    public void invalidPortProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        final String invalidValue = "invalid-value";
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + SetTransportListenerPort.IDENTIFIER_SHORT_OPTION, "an-identifier",
                    "-" + SetTransportListenerPort.PORT_SHORT_OPTION, "invalid-value" });
            fail("CommandInvalidArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, SetTransportListenerPort.PORT_SHORT_OPTION, invalidValue);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid transport listener identifier is provided,</li>
     * <li>a valid transport listener port is provided,</li>
     * <li>a valid component identifier is provided,</li>
     * <li>the BC Gateway is in state "STARTED"</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, the transport listener is correctly modified on the right component.
     * </p>
     */
    @Test
    public void componentIdentifierProvided_onStartedGateway() throws Exception {

        // Set the environment
        final String anotherComponentIdentifier = "component-B";
        final String transportListenerIdentifier = "tl-B";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        petalsBcGatewayRuntimeConfigurationMockOther.addTransportListener(transportListenerIdentifier,
                7501);
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient(anotherComponentIdentifier, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();
        assertEquals("Transport listener already configured on the default component", 0,
                petalsBcGatewayRuntimeConfigurationMockDefault.getTransportListeners().size());
        assertEquals("No transport listener already configured on the other component", 1,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners().size());

        // Execute command
        final Integer transportListenerNewPort = Integer.valueOf(45626);
        command.setShell(shell);
        command
                .execute(new String[] { "-" + SetTransportListenerPort.IDENTIFIER_SHORT_OPTION, transportListenerIdentifier,
                "-" + SetTransportListenerPort.PORT_SHORT_OPTION, transportListenerNewPort.toString(),
                "-" + SetTransportListenerPort.COMPONENT_ID_SHORT_OPTION, anotherComponentIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertEquals("Transport listener present on the default component", 0,
                petalsBcGatewayRuntimeConfigurationMockDefault.getTransportListeners().size());
        assertEquals("Transport listener not present on the right component", 1,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners().size());
        assertEquals("Unexpected transport listener identifier/port", transportListenerNewPort,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners()
                        .get(transportListenerIdentifier)[0]);
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid transport listener identifier is provided,</li>
     * <li>a valid transport listener port is provided,</li>
     * <li>a valid component identifier is provided,</li>
     * <li>the BC Gateway is in state "LOADED"</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, the transport listener is correctly modified on the right component.
     * </p>
     */
    @Test
    public void componentIdentifierProvided_onLoadedGateway() throws Exception {

        // Set the environment
        final String anotherComponentIdentifier = "component-B";
        final String transportListenerIdentifier = "tl-B";
        final PetalsBcGatewayInstallationMock petalsBcGatewayInstallationConfigurationServiceClientMockDefault = new PetalsBcGatewayInstallationMock();
        final PetalsBcGatewayInstallationMock petalsBcGatewayInstallationConfigurationServiceClientMockOther = new PetalsBcGatewayInstallationMock();
        petalsBcGatewayInstallationConfigurationServiceClientMockOther.addTransportListener(transportListenerIdentifier,
                7501);

        this.jmxApi.addComponentInstallerClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, petalsBcGatewayInstallationConfigurationServiceClientMockDefault);
        this.jmxApi.addComponentInstallerClient(anotherComponentIdentifier, ComponentType.BINDING,
                null, petalsBcGatewayInstallationConfigurationServiceClientMockOther);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();
        assertEquals("Transport listener already configured on the default component", 0,
                petalsBcGatewayInstallationConfigurationServiceClientMockDefault.getTransportListeners().size());
        assertEquals("No transport listener already configured on the other component", 1,
                petalsBcGatewayInstallationConfigurationServiceClientMockOther.getTransportListeners().size());

        // Execute command
        final Integer transportListenerNewPort = Integer.valueOf(45626);
        command.setShell(shell);
        command
                .execute(new String[] { "-" + SetTransportListenerPort.IDENTIFIER_SHORT_OPTION, transportListenerIdentifier,
                "-" + SetTransportListenerPort.PORT_SHORT_OPTION, transportListenerNewPort.toString(),
                "-" + SetTransportListenerPort.COMPONENT_ID_SHORT_OPTION, anotherComponentIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertEquals("Transport listener present on the default component", 0,
                petalsBcGatewayInstallationConfigurationServiceClientMockDefault.getTransportListeners().size());
        assertEquals("Transport listener not present on the right component", 1,
                petalsBcGatewayInstallationConfigurationServiceClientMockOther.getTransportListeners().size());
        assertEquals("Unexpected transport listener identifier/port", transportListenerNewPort,
                petalsBcGatewayInstallationConfigurationServiceClientMockOther.getTransportListeners()
                        .get(transportListenerIdentifier)[0]);
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-i' is given but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void transportListenerIdentifierValueMissing() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        final Command command = new SetTransportListenerPort();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + SetTransportListenerPort.IDENTIFIER_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, SetTransportListenerPort.IDENTIFIER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.set-transport-listener-port" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-p' is given but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void transportListenerPortValueMissing() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        final Command command = new SetTransportListenerPort();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + SetTransportListenerPort.PORT_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, SetTransportListenerPort.PORT_SHORT_OPTION);
        }
    }
}
